--  File:     ACID_TRAN.SQL
--            Microsoft TPC-H Benchmark Kit Ver. 2.17.0-1020
--
-- Implemented trunc(x,2) as ROUND(x,2,1)
--

if exists ( select name from sysobjects where name = 'acid_tran_com' )
	drop procedure acid_tran_com
go

create proc acid_tran_com 
			@o_key		bigint,
			@l_key		bigint,
			@delta		int
as
declare	@ototal		money,
	@new_ototal	money,
	@quantity	money,
	@extprice	money,
	@new_extprice	money,
	@pkey		bigint,
	@skey		bigint,
	@tax		money,
	@disc		money,
	@cost		money,
	@rprice		money

begin
declare @deadlockretries int -- number of retries in case of deadlock
set @deadlockretries = 60
while (@deadlockretries > 0)
	begin
		begin try

	begin tran T

		select	@ototal		= O_TOTALPRICE
		from	ORDERS
		where	O_ORDERKEY	= @o_key

		select	@quantity	= L_QUANTITY,
			@extprice	= L_EXTENDEDPRICE,
			@pkey		= L_PARTKEY,
			@skey		= L_SUPPKEY,
			@tax		= L_TAX,
			@disc		= L_DISCOUNT
		from	LINEITEM
		where	L_ORDERKEY	= @o_key and
			L_LINENUMBER	= @l_key	

		select	@ototal		= @ototal - ROUND((ROUND((@extprice * (1 - @disc)),2,1) * (1 + @tax)),2,1),
			@rprice		= ROUND((@extprice/@quantity),2,1),
			@cost		= ROUND((@rprice * @delta),2,1),
			@new_extprice	= @extprice + @cost,
			@new_ototal	= ROUND((@new_extprice * (1.0 - @disc)),2,1),
			@new_ototal	= ROUND((@new_ototal * (1.0 + @tax)),2,1),
			@new_ototal	= @ototal + @new_ototal

		update	LINEITEM
		set	L_EXTENDEDPRICE	= @new_extprice,
			L_QUANTITY	= @quantity + @delta
		where	L_ORDERKEY	= @o_key and
			L_LINENUMBER	= @l_key

		update	ORDERS
		set	O_TOTALPRICE	= @new_ototal
		where	O_ORDERKEY	= @o_key	

		insert into HISTORY_ACI values(	@pkey,
						@skey,
						@o_key,
						@l_key,
						@delta,
						getdate())

	commit transaction T
-- reset the deadlock counter so we fall through and return
			set	@deadlockretries = 0

		end try
        begin catch

			-- deadlock exception number is 1205 
			if ( error_number() = 1205 )  
				begin
					set @deadlockretries = @deadlockretries - 1  
				end
			-- other exceptions
			else
				begin
					set @deadlockretries = 0  
				end
    
            if xact_state() <> 0  
                rollback transaction T
        end catch
    end
	select	@rprice 'RPrice',
		@quantity 'Quantity',
		@tax 'Tax',
		@disc 'Discount',
		@extprice 'Extended Price',
		@ototal 'Total'

end

go


