--  File:	ACID_VALUES.SQL
--		Microsoft TPC-H Benchmark Kit Version 2.14.0-1010
--		Copyright Microsoft, 2010, 2011
--
--  Purpose:	This script creates a stored procedure that, when called with the 
--		number of desired streams, will populate the acid_values table.
--		The ACID procedures read this table to guarantee random and valid input.
--		This table includes a unique clustered index to guarantee that all values
--		are unique for each stage of the ACI tests.  If an attempt is made to insert
--		a value that currently exists in the table, the insert errors out and 
--		the values are recalculated and another insert is attempted.
--
--		The first 10 rows, seq_num, are for atomicity and isolation.  The remaining
--		x * 200 rows are for consistency and durability, where x is the number of streams desired.
--

if exists ( select name from sysobjects where name = 'tpch_acid_values' )
	drop procedure tpch_acid_values
go

create proc tpch_acid_values	@num_streams	int
							 
as

declare @acid_test	char(25),
        @rand_o_key	bigint,
        @rand_l_key	bigint,
        @rand_delta	int,
        @rand_part_key	bigint,
        @rand_supp_key	bigint,
        @cnt		bigint,
        @multiplier	bigint,
        @counter	bigint,
        @found_one	tinyint,
        @cons_rows	bigint

--	set initial values for loop controls
select  @found_one	= 0,
        @counter	= 1,
        @cnt		= 0,
        @cons_rows	= @num_streams * 200 

--	loop to populate table for ACID
while (@counter	<= (@cons_rows + 10) )
begin

    --	Set ACI test name based on counter
    select  @acid_test  = case	@counter
                                when 1 then 'Atomicity_Commit'
                                when 2 then 'Atomicity_Rollback'
                                when 3 then 'Isolation 1'
                                when 4 then 'Isolation 2'
                                when 5 then 'Isolation 3'
                                when 6 then 'Isolation 3'
                                when 7 then 'Isolation 4'
                                when 8 then 'Isolation 4'
                                when 9 then 'Isolation 5'
                                when 10 then 'Isolation 6'
                                else 'Consistency_Durability'
                          end

    --	loop here until valid values of O_KEY and L_KEY are found
    while (@found_one = 0)
    begin
            select	@rand_o_key	= ROUND((RAND()+ (RAND())*22),0,1)

			select	@multiplier	= case	@rand_o_key
							when   0 then 1
							when   1 then 10
							when   2 then 100
							when   3 then 1000
							when   4 then 10000
							when   5 then 100000
							when   6 then 1000000
							when   7 then 10000000
							when   8 then 100000000
							when   9 then 1000000000
							when  10 then 10000000000
							when  11 then 100000000000
							when  12 then 10000000000
							when  13 then 1000000000
							when  14 then 100000000
							when  15 then 10000000
							when  16 then 1000000
							when  17 then 100000
							when  18 then 10000
							when  19 then 1000
							when  20 then 100
							when  21 then 10
							when  22 then 1 
						  end

			if (@counter = 9)
			begin
				while	(@cnt = 0)
				begin
					select	@rand_part_key	= ROUND(((RAND() * (@multiplier+9))/5),0)
					
					select	@cnt		= count(*)
					from	PARTSUPP
					where	PS_PARTKEY	= @rand_part_key
					end

					set	rowcount @cnt

					select	@rand_supp_key	= PS_SUPPKEY
					from	PARTSUPP
					where	PS_PARTKEY	= @rand_part_key

					set	rowcount 0
				end
				else
				begin
					select	@rand_part_key	= 0,
                            @rand_supp_key	= 0
				end

				select	@rand_o_key	= ROUND((RAND() * @multiplier),0,1)

				select	@cnt		= (count(*) + 1)/2
				from	LINEITEM
				where	L_ORDERKEY	= @rand_o_key

			
				if (@cnt > 0) select @found_one = 1
			end

		set	rowcount @cnt

		select	TOP 1
                @rand_l_key	= L_LINENUMBER
		from	LINEITEM
		where	L_ORDERKEY	= @rand_o_key

		set	rowcount 0

		select	@rand_delta	= (RAND()*100)

		select	@cnt		= count(*)
		from	acid_values
		where	acid_o_key	= @rand_o_key and
				acid_test	= 'Consistency_Durability'
		
		if (@cnt = 0)
		begin
		--	Insert @rand_o_key, @rand_o_key, and @rand_delta into acid_values table
			insert	into acid_values values(@acid_test,
											@rand_o_key,
											@rand_l_key,
											@rand_delta,
											@rand_part_key,
											@rand_supp_key,
											@counter)

		--	If a duplicate was found, repeat the search for valid keys
		--	@error 2601 indicates an attempt to insert a duplicate
		if (@@error = 2601)
			begin
				select	@counter	= @counter,
						@found_one	= 0,
						@cnt		= 0
			end
		else		
			begin
				select	@counter	= @counter + 1,
						@found_one	= 0,
						@cnt		= 0
			end
		end
		else
		begin
			select	@counter	= @counter,
					@found_one	= 0,
					@cnt		= 0
		end
	end



