Attribute VB_Name = "ConnDtls"
'  FILE:       ConnDtls.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functionality to add, modify, delete
'              and edit connections. Also other connection-specific
'              code.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

Private mcQueryConnDtl As New cConnDtl

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = ".ConnDtls."

Public Function CopyConnDtl(ByVal NodeIdentifier As Long, _
        Optional ByVal NodeKey As String = gstrEmptyString) As Object

    ' Returns an object populated with the data for the passed in identifier
    
    On Error GoTo CopyConnDtlErr

    Set CopyConnDtl = gcConnDtls.QueryConnDtl(NodeIdentifier)
            
    Exit Function
    
CopyConnDtlErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Set CopyConnDtl = Nothing
    Err.Raise vbObjectError + errCopyFailed, _
            mstrModuleName, LoadResString(errCopyFailed)
    
End Function
Public Sub CreateConnDtl(ByVal cNewConnDtl As cConnDtl, _
        ByVal lngWorkspaceId As Long)
    
    ' Creates a new ConnDtl record with the properties
    ' same as the those of the passed in object
    
    Dim cConnDtlToCreate As cConnDtl
    
    On Error GoTo CreateConnDtlErr
    
    Set cConnDtlToCreate = cNewConnDtl.Clone
    cConnDtlToCreate.WorkspaceId = lngWorkspaceId
    
    ' Call a procedure to save the record
    Call InsertConnDtl(cConnDtlToCreate)

    Set cConnDtlToCreate = Nothing
    
    Exit Sub
    
CreateConnDtlErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errCreateConnectionFailed, _
            mstrModuleName & "CreateConnDtl", _
            LoadResString(errCreateConnectionFailed)

End Sub

Public Function GetConnectionTypeString(iConnType As ConnectionType) As String

    GetConnectionTypeString = IIf(iConnType = ConnTypeStatic, gsConnTypeStatic, gsConnTypeDynamic)
    
End Function

Private Sub InsertConnDtl(ByVal cNewConnDtl As cConnDtl)
    
    Dim iAppend As Integer
    
    On Error GoTo InsertConnDtlErr

    iAppend = 0
    gcConnDtls.Add cNewConnDtl
    
    ' Load ConnDtl values in the tree and list view controls
    Call AddNode(frmMain.treStepView, frmMain.lvwStepView, _
            cNewConnDtl.ConnNameId, cNewConnDtl.ConnName, _
            cNewConnDtl.WorkspaceId, gintConnectionDtl, cNewConnDtl)
    
    ' Set the tag for the workspace node to indicate that
    ' it has been modified.
    Call WspModified(cNewConnDtl.WorkspaceId)
    
    Exit Sub
    
InsertConnDtlErr:
    ' If the error raised by the add function is due to a duplication
    ' of the ConnDtl name, then try to generate a unique name
    If Err.Number - vbObjectError = errDupConnDtlName Then
        iAppend = iAppend + 1
        cNewConnDtl.ConnName = cNewConnDtl.ConnName & CStr(iAppend)
        ' Try to insert the record again
        Resume
    End If
    
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errCreateConnectionFailed, _
            mstrModuleName, LoadResString(errCreateConnectionFailed)

End Sub
Sub ConnDtlOK()

    On Error GoTo ConnDtlOKErr

    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    If frmConnDtl.Tag = gstrUpdateMode Then
        Call ConnDtlModify
    Else
        Call ConnDtlNew
    End If
    
    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
ConnDtlOKErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Reset the mousepointer
    Call ShowFree
    
End Sub

Private Sub PopulateConnections(cboList As ComboBox, lWsp As Long, _
        Optional sDefault As String = gstrEmptyString)

    ' Populates the passed in combo box with all connection strings that have been
    ' defined for the workspace
    Dim lIndex As Long
    
    cboList.Clear
    
    For lIndex = 0 To gcConnections.Count - 1
        If gcConnections(lIndex).WorkspaceId = lWsp And _
                gcConnections(lIndex).IndOperation <> DeleteOp Then
            
            cboList.AddItem gcConnections(lIndex).ConnectionName
        End If
    Next lIndex
    
    ' Set the default text to the passed in value
    If StringEmpty(sDefault) Then
        If cboList.ListCount > 0 Then cboList.ListIndex = 0
    Else
        cboList.Text = sDefault
    End If

    Exit Sub
    
End Sub

Sub ShowModifyConnDtlForm(Optional ByVal lngIdentifier As Long = 0)

    Dim lngConnId As Long
    
    On Error GoTo ShowModifyConnDtlFormErr

    If lngIdentifier = 0 Then
        lngConnId = IdentifierSelected
    Else
        lngConnId = lngIdentifier
    End If

    BugAssert lngConnId <> 0, "ConnDtlModify called when an item wasn't selected!"
    Set mcQueryConnDtl = gcConnDtls.QueryConnDtl(lngConnId)
    
    Call PopulateConnections(frmConnDtl.cboConnect, mcQueryConnDtl.WorkspaceId, _
            mcQueryConnDtl.ConnectionString)
    
    Call CopyConnClassToForm(mcQueryConnDtl)
    
    frmConnDtl.Tag = gstrUpdateMode
    
    ' Reset the mousepointer
    ShowFree
    
    frmConnDtl.Show vbModal, frmMain
    
    Exit Sub
    
ShowModifyConnDtlFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errShowFormFailed, _
            mstrModuleName & "ShowModifyConnDtlForm", LoadResString(errShowFormFailed)
    
End Sub

Sub ShowCreateConnDtlForm(Optional ByVal lngWorkspaceId As Long = 0)

    Dim lngWorkspace As Long
    
    On Error GoTo ShowCreateConnDtlFormErr

    If lngWorkspaceId = 0 Then
        lngWorkspace = WorkspaceSelected
    Else
        lngWorkspace = lngWorkspaceId
    End If
    
    frmConnDtl.Tag = gstrInsertMode
    
    Call PopulateConnections(frmConnDtl.cboConnect, lngWorkspace)
    Call SetOption(frmConnDtl.optConnType, giDefaultConnType)
    
    ' Store the workspace identifier as a hidden field on the form.
    frmConnDtl.lblWorkspaceId = Str(lngWorkspace)
    
    ' Reset the mousepointer
    ShowFree
    
    frmConnDtl.Show vbModal, frmMain
    
    Exit Sub
    
ShowCreateConnDtlFormErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errShowFormFailed, _
            mstrModuleName & "ShowCreateConnDtlForm", LoadResString(errShowFormFailed)
    
End Sub
Public Sub ConnDtlModify()

    Dim cNewConnDtl As cConnDtl
    
    On Error GoTo ConnDtlModifyErr

    Set cNewConnDtl = mcQueryConnDtl.Clone
    
    Call CopyConnFormToClass(cNewConnDtl)
    
    ' Modify the parameter record in the array
    gcConnDtls.Modify cNewConnDtl
    
    ' Modify parameter values in the list and tree view controls
    Call UpdateView(frmMain.treStepView, frmMain.lvwStepView, _
        cNewConnDtl.ConnNameId, cNewConnDtl.ConnName, _
        cNewConnDtl, gintConnectionDtl)

    ' Set the tag for the workspace node to indicate that
    ' it has been modified.
    Call WspModified(cNewConnDtl.WorkspaceId)
    
    Set cNewConnDtl = Nothing
    
    ' Now that we're done, unload the form
    Unload frmConnDtl
    Exit Sub
    
ConnDtlModifyErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ShowError errConnectionUpdateFailed
    
End Sub
Public Sub ConnDtlDelete(Optional ByVal cConnToDelete As cConnDtl, _
        Optional ByVal lngIdentifier As Long = 0)

    Dim lngConnId As Long
    
    On Error GoTo ConnDtlDeleteErr
    
    If cConnToDelete Is Nothing And lngIdentifier = 0 Then
        lngConnId = IdentifierSelected
    Else
        lngConnId = lngIdentifier
    End If
    
    ' Call a procedure to delete the ConnDtl record
    Call DeleteConnDtlRecord(ConnDtl:=cConnToDelete, Identifier:=lngConnId)
    
    Exit Sub
    
ConnDtlDeleteErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ConnDtlDelete"
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteConnectionFailed, _
            gstrSource, _
            LoadResString(errDeleteConnectionFailed)
    
End Sub

Public Sub DeleteConnDtlRecord(Optional ByVal ConnDtl As cConnDtl, _
    Optional ByVal Identifier As Long = 0, _
    Optional ConnDtlKey As String = gstrEmptyString)
    ' This procedure calls all the routines necessary to delete
    ' a ConnDtl record
    
    Dim cConnToDelete As cConnDtl
    Dim lngConnId As Long
    
    On Error GoTo DeleteConnDtlRecordErr
    
    If Not ConnDtl Is Nothing Then
        Set cConnToDelete = ConnDtl
        lngConnId = cConnToDelete.ConnNameId
    ElseIf Identifier > 0 Then
        lngConnId = Identifier
        Set cConnToDelete = gcConnDtls.QueryConnDtl(lngConnId)
    ElseIf ConnDtlKey <> gstrEmptyString Then
        lngConnId = MakeIdentifierValid(ConnDtlKey)
        Set cConnToDelete = gcConnDtls.QueryConnDtl(lngConnId)
    Else
        On Error GoTo 0
        Err.Raise vbObjectError + errMandatoryParameterMissing, _
                mstrModuleName & "DeleteConnDtlRecord", _
                LoadResString(errMandatoryParameterMissing)
    End If
        
    gcConnDtls.Delete lngConnId

    ' Delete ConnDtl node in the list and tree view controls
    Call DeleteViewNode(frmMain.treStepView, frmMain.lvwStepView, _
        lngConnId, gintConnectionDtl, cConnToDelete.WorkspaceId)
    
    ' Set the tag for the workspace node to indicate that
    ' it has been modified.
    Call WspModified(cConnToDelete.WorkspaceId)
    
    Set cConnToDelete = Nothing
    Exit Sub
    
DeleteConnDtlRecordErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteConnectionFailed, _
            mstrModuleName, _
            LoadResString(errDeleteConnectionFailed)
    
End Sub
Public Sub ConnDtlNew()

    Dim cNewConnDtl As cConnDtl
    
    On Error GoTo ConnDtlNewErr

    ' All checks for validity of the ConnDtl properties values will be done
    ' by the ConnDtl class
        
    Set cNewConnDtl = New cConnDtl
    
    Call CopyConnFormToClass(cNewConnDtl)
    
    ' Call a procedure to save the parameter record
    Call InsertConnDtl(cNewConnDtl)
    
    Set cNewConnDtl = Nothing
    
    ' Now that we're done, unload the parameters form
    Unload frmConnDtl
    
    Exit Sub
    
ConnDtlNewErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    ShowError errCreateConnectionFailed

End Sub
Private Sub CopyConnClassToForm(cConn As cConnDtl)
    
    ' Store the workspace identifier as a hidden field on the form.
    frmConnDtl.lblWorkspaceId = Str(cConn.WorkspaceId)
    
    frmConnDtl.txtConnName = cConn.ConnName
    frmConnDtl.cboConnect.Text = cConn.ConnectionString
    Call SetOption(frmConnDtl.optConnType, cConn.ConnType)

End Sub
Private Sub CopyConnFormToClass(ByRef cConn As cConnDtl)

    cConn.WorkspaceId = Val(frmConnDtl.lblWorkspaceId)
    
    cConn.ConnName = frmConnDtl.txtConnName
    cConn.ConnectionString = frmConnDtl.cboConnect.Text
    cConn.ConnType = GetOption(frmConnDtl.optConnType)

End Sub
