Attribute VB_Name = "ODBCConn"
'  FILE:       ODBCConn.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functions to help build the connection
'              string using the frmODBCLogon form
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors are raised by this module
Private Const mstrModuleName As String = "ODBCConn."

Private msConnect As String
Private mlWspId As Long

Private Const msConnSep As String = ";"
Private Const msDriver As String = "DRIVER="
Private Const msServer As String = "SERVER="
Private Const msUID As String = "UID="
Private Const msPWD As String = "PWD="
Private Const msDB As String = "DATABASE="
Private Const msDSN As String = "DSN="

Private Declare Function SQLDataSources Lib "ODBC32.DLL" (ByVal henv&, ByVal fDirection%, ByVal szDSN$, ByVal cbDSNMax%, pcbDSN%, ByVal szDescription$, ByVal cbDescriptionMax%, pcbDescription%) As Integer
Private Declare Function SQLAllocEnv% Lib "ODBC32.DLL" (env&)

Private Const SQL_SUCCESS As Long = 0
Private Const SQL_FETCH_NEXT As Long = 1
Public Function CreateConnectString(sOldConnect As String, lWspId As Long) As String
    Dim iPosition As Integer
    
    On Error GoTo CreateConnectStringErr
    
    msConnect = sOldConnect
    
    mlWspId = lWspId
    
    ' Load all existing drivers and dsns
    Call GetDSNsAndDrivers
    
    ' Initialize the old values of the connect parameters
    If Not StringEmpty(sOldConnect) Then
        frmODBCLogon.txtDatabase = GetConnectParameter(sOldConnect, msDB)
        frmODBCLogon.txtPWD = GetConnectParameter(sOldConnect, msPWD)
        frmODBCLogon.txtServer = GetConnectParameter(sOldConnect, msServer)
        frmODBCLogon.txtUID = GetConnectParameter(sOldConnect, msUID)
        iPosition = FindComboEntry(frmODBCLogon.cboDrivers, _
                GetConnectParameter(sOldConnect, msDriver))
        If iPosition <> -1 Then
            frmODBCLogon.cboDrivers.ListIndex = iPosition
        End If
        iPosition = FindComboEntry(frmODBCLogon.cboDSNList, _
                GetConnectParameter(sOldConnect, msDSN))
        If iPosition <> -1 Then
            frmODBCLogon.cboDSNList.ListIndex = iPosition
        End If
    End If
    
    frmODBCLogon.Show vbModal, frmMain
    CreateConnectString = msConnect
    
    Exit Function

CreateConnectStringErr:
    LogErrors Errors
    Resume Next
    
End Function
Private Sub GetDSNsAndDrivers()
    On Error Resume Next
    
    Dim i As Integer
    Dim sDSNItem As String * 1024
    Dim sDRVItem As String * 1024
    Dim sDSN As String
    Dim sDRV As String
    Dim iDSNLen As Integer
    Dim iDRVLen As Integer
    Dim lHenv As Long         'handle to the environment

    frmODBCLogon.cboDSNList.AddItem "(None)"

    'get the DSNs
    If SQLAllocEnv(lHenv) <> -1 Then
        Do Until i <> SQL_SUCCESS
            sDSNItem = Space(1024)
            sDRVItem = Space(1024)
            i = SQLDataSources(lHenv, SQL_FETCH_NEXT, sDSNItem, 1024, iDSNLen, sDRVItem, 1024, iDRVLen)
            sDSN = VBA.Left(sDSNItem, iDSNLen)
            sDRV = VBA.Left(sDRVItem, iDRVLen)
                
            If sDSN <> Space(iDSNLen) Then
                frmODBCLogon.cboDSNList.AddItem sDSN
                frmODBCLogon.cboDrivers.AddItem sDRV
            End If
        Loop
    End If
    'remove the dupes
    If frmODBCLogon.cboDSNList.ListCount > 0 Then
        With frmODBCLogon.cboDrivers
            If .ListCount > 1 Then
                i = 0
                While i < .ListCount
                    If .List(i) = .List(i + 1) Then
                        .RemoveItem (i)
                    Else
                        i = i + 1
                    End If
                Wend
            End If
        End With
    End If
    frmODBCLogon.cboDSNList.ListIndex = 0
    
End Sub
Private Function GetConnectParameter(sConnect As String, sToken As String) As String
    ' Gets the value of the passed in parameter
    Dim iCurrent As Integer
    Dim iSeparator As Integer
    
    GetConnectParameter = gstrEmptyString

    iSeparator = InStr(sConnect, sToken)
    If iSeparator > 0 Then

        iCurrent = InStr(iSeparator, sConnect, msConnSep)
        If iCurrent = 0 Then
            iCurrent = Len(sConnect)
        End If
        
        GetConnectParameter = Mid$(sConnect, iSeparator + Len(sToken), iCurrent - iSeparator - Len(sToken))
    End If

End Function

Private Function GetConnectString() As String
    Dim sConnect As String
    Dim sDSN As String

    If frmODBCLogon.cboDSNList.ListIndex > 0 Then
        sDSN = frmODBCLogon.cboDSNList.Text
    Else
        sConnect = sConnect & msDriver & frmODBCLogon.cboDrivers.Text & msConnSep
        sConnect = sConnect & msServer & frmODBCLogon.txtServer.Text & msConnSep
    End If
    
    sConnect = sConnect & msUID & frmODBCLogon.txtUID.Text & msConnSep
    sConnect = sConnect & msPWD & frmODBCLogon.txtPWD.Text & msConnSep
    
    If Len(frmODBCLogon.txtDatabase.Text) > 0 Then
        sConnect = sConnect & msDB & frmODBCLogon.txtDatabase.Text & msConnSep
    End If
    
    If Len(sDSN) > 0 Then
        sConnect = sConnect & msDSN & sDSN & msConnSep
    End If
    
    GetConnectString = sConnect

End Function
Public Sub ConnectTest()

    If Test Then
        ShowMessage errConnected
    Else
        ShowError errConnectFailed
    End If
    
End Sub
Private Function Test() As Boolean

    Dim cTempRDOEnv As rdoEnvironment
    Dim cTempRDOConn As rdoConnection
    Dim strTest As String
    
    On Error Resume Next
    
    ShowBusy
    
    strTest = GetConnectString()
    
    Set cTempRDOEnv = gRunEngine.rdoEnvironments(0)
    Set cTempRDOConn = cTempRDOEnv.OpenConnection(dsname:="", prompt:=rdDriverNoPrompt, _
            Connect:=SubstituteParameters(strTest, mlWspId))
    Test = Not cTempRDOConn Is Nothing
    If Test Then
        cTempRDOConn.Close
        cTempRDOEnv.Close
    End If
    
    ShowFree
    
End Function

Public Sub ConnectOk()

    On Error GoTo ConnectOkErr
    
    If Not Test Then
        If Confirm(conSaveConnect, contSaveConnect, Buttons:=vbYesNo) Then
            msConnect = GetConnectString()
            Unload frmODBCLogon
        End If
    Else
        msConnect = GetConnectString()
        Unload frmODBCLogon
    End If
    
    Exit Sub

ConnectOkErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)

End Sub
Public Sub ConnectCancel()
    Unload frmODBCLogon
End Sub

