Attribute VB_Name = "StepType"
'  FILE:       StepType.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    This module displays the type of steps that the user can create.
'              The main function is GetStepTypeAndPosition which returns the choice
'              made by the user
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "StepType."

Private mintStepType As Integer
Private mlParentStepId As Long
Private msParentVerNo As String
Private miLevel As Integer
Private miSequence As Integer
Private mstrParentStepKey As String

Private mbOk As Boolean

' Must be in sync with the index values for the option group, optInsert
' on the Step Type form
Private Enum InsertPosition
    gintInsertAfter = 1
    gintInsertBefore
End Enum

Private Const mstrPosition As String = "Select Step Position"
Public Sub GetStepPositionDtls(ByRef cStepRec As cStep)
    ' Populates the step record with the step position details

    If mintStepType <> gintNoOption Then
        cStepRec.StepType = mintStepType
    End If
    
    cStepRec.ParentStepId = mlParentStepId
    cStepRec.ParentVersionNo = msParentVerNo
    cStepRec.StepLevel = miLevel
    cStepRec.SequenceNo = miSequence
    
End Sub
Private Sub ResetStepPositionDtls()
    ' Resets the step position details

    mlParentStepId = 0
    msParentVerNo = gstrMinVersion
    miLevel = gintMinLevel
    miSequence = gintMinSequenceNo
    
End Sub

Public Function GetStepTypeAndPosition(strParentKey As String) As Integer

    On Error GoTo GetStepTypeAndPositionErr
    gstrSource = mstrModuleName & "GetStepTypeAndPosition"
    
    mintStepType = gintNoOption
    
    Call SetOption(frmStepTypeAndPos.optInsert, gintInsertAfter)
    
    Call LoadChildNodes(strParentKey)
    
    ' Reset the mousepointer
    ShowFree
    
    frmStepTypeAndPos.Show vbModal, frmMain
    GetStepTypeAndPosition = mintStepType
    
    Exit Function
    
GetStepTypeAndPositionErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "GetStepTypeAndPosition"
    Err.Raise vbObjectError + errGetStepTypeAndPositionFailed, _
            gstrSource, _
            LoadResString(errGetStepTypeAndPositionFailed)
    
End Function
Public Function GetStepPosition(ByRef cStepToInit As cStep, _
        sParentKey As String) As Boolean

    On Error GoTo GetStepPositionErr
    gstrSource = mstrModuleName & "GetStepPosition"
    
    mbOk = True
    Call DisableStepTypeFields
    frmStepTypeAndPos.Caption = mstrPosition
    
    Call SetOption(frmStepTypeAndPos.optInsert, gintInsertAfter)
    
    Call LoadChildNodes(sParentKey)
    
    ' Reset the mousepointer
    ShowFree
    
    frmStepTypeAndPos.Show vbModal, frmMain
    
    If mbOk Then
        Call GetStepPositionDtls(cStepToInit)
    End If
    
    GetStepPosition = mbOk
    Exit Function
    
GetStepPositionErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "GetStepPosition"
    Err.Raise vbObjectError + errGetStepTypeAndPositionFailed, _
            gstrSource, _
            LoadResString(errGetStepTypeAndPositionFailed)
    
End Function
Private Sub DisableStepTypeFields()

    On Error GoTo DisableStepTypeFieldsErr
    
    SetEnabledForCtlArray frmStepTypeAndPos.optStepType, False
    frmStepTypeAndPos.fraStepType.Enabled = False
    
    Exit Sub
    
DisableStepTypeFieldsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Resume Next
    
End Sub
Public Sub InsertPositionSelected(Index As Integer)
    Dim strSelected As String

    On Error GoTo InsertPositionSelectedErr
    
    If (Index = gintInsertBefore) Then
        ' Force the option to InsertAfter if the root node is selected
        strSelected = ItemSelectedInTree(frmStepTypeAndPos.trePosition)
        If frmStepTypeAndPos.trePosition.Nodes(strSelected).Root.Key = strSelected Then
            SetOption frmStepTypeAndPos.optInsert, gintInsertAfter
        End If
    End If

    Exit Sub
    
InsertPositionSelectedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    frmStepTypeAndPos.trePosition.Nodes.Clear
    
    ' Tell the user that the node will be created as the last child of the selected node
    ShowError (errDefaultPosition)
    
End Sub

Private Sub LoadChildNodes(strParentKey As String)

    Dim strChild As String
    Dim nodChild As comctllib.Node
    
    On Error GoTo LoadChildNodesErr
    gstrSource = mstrModuleName & "LoadChildNodes"
    
    frmStepTypeAndPos.trePosition.Nodes.Clear
    
    Set nodChild = frmMain.treStepView.Nodes(strParentKey)
    frmStepTypeAndPos.trePosition.Nodes.Add , , strParentKey, nodChild.Text, nodChild.Image, nodChild.SelectedImage
    
    If frmMain.treStepView.Nodes(strParentKey).Children > 0 Then
        strChild = frmMain.treStepView.Nodes(strParentKey).Child.Key
        
        Do
            Set nodChild = frmMain.treStepView.Nodes(strChild)
            frmStepTypeAndPos.trePosition.Nodes.Add strParentKey, tvwChild, _
                    strChild, nodChild.Text, nodChild.Image, nodChild.SelectedImage
        
            If nodChild.LastSibling.Key <> strChild Then
                strChild = nodChild.Next.Key
            Else
                ' Select this node by default
                frmStepTypeAndPos.trePosition.Nodes(strChild).Selected = True
                frmStepTypeAndPos.trePosition.Nodes(strChild).EnsureVisible
                strChild = gstrEmptyString
            End If
        
        Loop While Not StringEmpty(strChild)
    Else
        frmStepTypeAndPos.trePosition.Nodes(strParentKey).Selected = True
    End If
    
    Exit Sub
    
LoadChildNodesErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    frmStepTypeAndPos.trePosition.Nodes.Clear
    
    ' Tell the user that the node will be created as the last child of the selected node
    ShowError errDefaultPosition
    
End Sub
Public Sub StepSelected(Node As comctllib.Node)

    On Error GoTo StepSelectedErr
    gstrSource = mstrModuleName & "StepSelected"
    
    If Node.Key = Node.Root.Key Then
        Call SetOption(frmStepTypeAndPos.optInsert, gintInsertAfter)
    End If
    
    Exit Sub
    
StepSelectedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    frmStepTypeAndPos.trePosition.Nodes.Clear
    
    ' Tell the user that the node will be created as the last child of the selected node
    ShowError (errDefaultPosition)
    
End Sub
Public Sub StepTypeCancel()

    On Error GoTo StepTypeCancelErr
    
    mbOk = False
    Call ShowBusy
    
    mintStepType = gintNoOption
    Call ResetStepPositionDtls
    
    Unload frmStepTypeAndPos

    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
StepTypeCancelErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Resume Next

End Sub

Public Sub StepTypeOK()
    Dim sSibling As String
    Dim sSelected As String
    Dim cSibStep As cStep
    Dim cParent As cStep
    Dim iPosition As InsertPosition

    On Error GoTo StepTypeOKErr
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    If frmStepTypeAndPos.fraStepType.Enabled Then
        mintStepType = GetOption(frmStepTypeAndPos.optStepType)
    End If
    
    If frmStepTypeAndPos.trePosition.Nodes.Count <= 0 Then
        ' An error occurred at some stage - reset the position details and exit
        Call ResetStepPositionDtls
    Else
    
        sSelected = ItemSelectedInTree(frmStepTypeAndPos.trePosition)
        iPosition = GetOption(frmStepTypeAndPos.optInsert)
                
        mstrParentStepKey = frmStepTypeAndPos.trePosition.Nodes(sSelected).Root.Key
        If IsLabel(mstrParentStepKey) Then
            mlParentStepId = 0
            msParentVerNo = gstrMinVersion
        Else
            mlParentStepId = MakeIdentifierValid(mstrParentStepKey)
            Set cParent = gcSteps.QueryStep(mlParentStepId)
            msParentVerNo = cParent.VersionNo
        End If
        
        If sSelected = frmStepTypeAndPos.trePosition.Nodes(sSelected).Root.Key Then
            ' The node will be added as the first child
            If mlParentStepId > 0 Then
                miLevel = cParent.StepLevel + 1
            Else
                miLevel = gintMinLevel
            End If
            miSequence = gintMinSequenceNo
        Else
            ' Has to be a worker/manager step
            Set cSibStep = gcSteps.QueryStep(MakeIdentifierValid(sSelected))
            miLevel = cSibStep.StepLevel
            If iPosition = gintInsertBefore Then
                miSequence = cSibStep.SequenceNo
            Else
                miSequence = cSibStep.SequenceNo + 1
            End If
            Set cSibStep = Nothing
        End If
        BugAssert miSequence >= gintMinSequenceNo, "Sequence # is invalid"
        
        Set cParent = Nothing
    End If
    
    Unload frmStepTypeAndPos

    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
StepTypeOKErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Try to create the new step as the last child of selected step
    Call ResetStepPositionDtls
    ShowError errDefaultPosition
    
End Sub
