Attribute VB_Name = "WorkspaceSM"
'  FILE:       WorkspaceSM.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    This module contains all the user-interface processing for
'              the workspace entity
'              It is called WorkspaceSM, since Workspace is a standard Visual
'              Basic object and we want to avoid any confusion with it.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Constants for the text that will be displayed at the static
' label nodes
Public Const mstrWorkspaceLabelText = "Workspaces: "
Private Const mstrGlobalsLabelText = "Globals "
Private Const mstrStepLabelText = "Steps "
Private Const mstrGlobalStepLabelText = "Global Steps "
Private Const mstrParameterLabelText = "Parameters "
Private Const mstrBuiltInLabelText = "Built-in Parameters "
Private Const mstrConnectStringLabelText = "Connection Strings "
Private Const mstrConnectDtlLabelText = "Connections "
Private Const mstrApplicationLabelText = "Application Extensions "

Private Const msOpenWsp = "Open Workspace"
Private Const msImportWsp = "Import Workspace"
Private Const msExportWsp = "Export Workspace"
Public Const msAppExtensionCaption = "Shell Application"

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "WorkspaceSM."

Const mintStepRecord As Integer = 0
Const mintParameterRecord As Integer = 1
Private Sub DelChildParamNodes(strParent As String)
    Dim nodChild As Node

    ' While more parameter nodes exist
    While frmMain.treStepView.Nodes(strParent).Children > 0
        Set nodChild = frmMain.treStepView.Nodes(strParent).Child
        
        ' Call a procedure to carry out the processing necessary
        ' to delete the parameter from the db and the loaded workspace
        Call DeleteParameterRecord(ParameterKey:=nodChild.Key)
    Wend

End Sub

Public Sub ShowDatabaseFile()

    On Error GoTo ShowDatabaseFileErr
    
    Load frmWorkspace
    frmWorkspace.Tag = gstrPropertiesMode
    frmWorkspace.Caption = "Database Properties"
    frmWorkspace.lblWorkspaceName = "Currently Open Database: "
    frmWorkspace.txtWorkspaceName = dbsAttTool.Name
    frmWorkspace.txtWorkspaceName.Locked = True
    
    frmWorkspace.Show vbModal, frmMain
    
    Exit Sub

ShowDatabaseFileErr:
    Call LogErrors(Errors)
    
    Call ShowError(errDisplayPropertiesFailed)

End Sub
Public Function ConfirmAndSaveChanges(ByVal lngWorkspaceId As Long) As Boolean

    ' Saves changes to the workspace after displaying a confirmation
    ' message to the user
    
    Dim strWspName As String
    Dim strWspKey As String
    
    On Error GoTo ConfirmAndSaveChangesErr
    
    BugAssert lngWorkspaceId <> 0, "Invalid Workspace Id"
    
    ConfirmAndSaveChanges = False
    
    strWspKey = MakeKeyValid(lngWorkspaceId, gintWorkspace)
    If IsWspModified(strWspKey) Then
        strWspName = FilterWorkspaceName(frmMain.treStepView.Nodes(strWspKey).Text)
        
        ' Make sure that the user really wants to save changes
        If Confirm(conSave, contSave, TitleParameter:=strWspName, Buttons:=vbYesNo) Then
            Call SaveWorkspace(lngWorkspaceId)
            ConfirmAndSaveChanges = True
        End If
    Else
        ConfirmAndSaveChanges = True
    End If
    
    Exit Function
    
ConfirmAndSaveChangesErr:
    ' Log the VB error code
    LogErrors Errors
    gstrSource = mstrModuleName & "ConfirmAndSaveChanges"
    On Error GoTo 0
    Err.Raise vbObjectError + errSaveWorkspaceFailed, gstrSource, _
            LoadResString(errSaveWorkspaceFailed)

End Function
Private Sub LoadRecordsetInTreeView(treTreeView As TreeView, _
    rstRecordsInWorkSpace As Recordset, _
    intRecordSetType As Integer)

    Dim nodCurLevelNode As Node
    Dim strNodeText As String
    Dim strParentKey As String
    Dim intNodeType As Integer
    Dim intParentNodeType As String
    Dim intImageIndex As Integer
    Dim intExpImageIndex As Integer
            
    On Error GoTo LoadRecordsetInTreeViewErr
    
    ' This function will load the passed in treTreeView control
    ' with all the records in the recordset, which may contain either
    ' parameter or step records
    ' The 0th field will form the key of each node
    ' For all nodes other than step nodes, the 1st field will be stored as the text
    ' The 4th field will be stored as the tag of the node, if necessary
    
    If rstRecordsInWorkSpace.RecordCount = 0 Then
        Exit Sub
    End If
    
    rstRecordsInWorkSpace.MoveFirst
    While Not rstRecordsInWorkSpace.EOF
        ' If the recordset contains step records, each of the
        ' records may be either a global step or a non-global
        ' step.
        ' Hence each record will need to be checked for whether
        ' the step being loaded is global
        ' Modify the node type accordingly
        If intRecordSetType = mintStepRecord Then
            intNodeType = TypeOfStep(StepRecord:=rstRecordsInWorkSpace)
        ElseIf intRecordSetType = gintNodeParamConnection Or _
                intRecordSetType = gintConnectionDtl Then
            intNodeType = intRecordSetType
        Else
            intNodeType = gintParameter + rstRecordsInWorkSpace!parameter_type
        End If
        
        ' Find the text for the current node
        strNodeText = GetNodeText(rstRecordsInWorkSpace, intNodeType)
        If StringEmpty(strNodeText) Then
            ' Error already raised by called function
            Exit Sub
        End If
        
        ' Get the parent key for the current record - it may
        ' be a label node or a step node
        strParentKey = GetParentKey(intNodeType, rstRecordsInWorkSpace)
            
        If intRecordSetType = mintStepRecord Then
            intImageIndex = GetCollapsedIcon(intNodeType, bEnabled:=rstRecordsInWorkSpace.Fields("enabled_flag"))
            intExpImageIndex = GetExpandedIcon(intNodeType, bEnabled:=rstRecordsInWorkSpace.Fields("enabled_flag"))
        Else
            intImageIndex = GetCollapsedIcon(intNodeType)
            intExpImageIndex = GetExpandedIcon(intNodeType)
        End If

        ' Add the current node as a child of the parent
        Set nodCurLevelNode = treTreeView.Nodes.Add(strParentKey, tvwChild, _
            MakeKeyValid(rstRecordsInWorkSpace(0).Value, intNodeType), _
            strNodeText, intImageIndex)
        nodCurLevelNode.ExpandedImage = intExpImageIndex
        
        If intRecordSetType = mintStepRecord Then
            nodCurLevelNode.Tag = rstRecordsInWorkSpace(4)
        End If
        
        rstRecordsInWorkSpace.MoveNext
    Wend
    
    Exit Sub
    
LoadRecordsetInTreeViewErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "LoadRsInTree"
    On Error GoTo 0
    WriteError errLoadRsInTreeFailed, gstrSource, _
            "Node Text: " & strNodeText
    Err.Raise vbObjectError + errLoadRsInTreeFailed, _
            gstrSource, _
            LoadResString(errLoadRsInTreeFailed)
    
End Sub
Private Function GetParentKey(intTypeOfNode As Integer, _
    rstRecords As Recordset) As String
    ' Returns the key for the parent node depending on the
    ' type of node passed in

    Dim strParentKey As String
    
    On Error GoTo GetParentKeyErr
    
    Select Case intTypeOfNode
        Case gintGlobalStep
            GetParentKey = LabelGlobalStep(rstRecords![workspace_id])
        Case gintManagerStep, gintWorkerStep
            ' Only a manager step can have sub-steps associated
            ' with it. Hence the parent will be a manager step.
            If rstRecords![parent_step_id] > 0 Then
                GetParentKey = MakeKeyValid( _
                    rstRecords![parent_step_id], gintManagerStep)
            Else
                GetParentKey = LabelStep(rstRecords![workspace_id])
            End If
        Case gintParameter
            GetParentKey = LabelParameter(rstRecords![workspace_id])
        Case gintNodeParamConnection
            GetParentKey = LabelParamConnection(rstRecords![workspace_id])
        Case gintNodeParamExtension
            GetParentKey = LabelAppExtension(rstRecords![workspace_id])
        Case gintNodeParamBuiltIn
            GetParentKey = LabelBuiltIns(rstRecords![workspace_id])
        Case gintConnectionDtl
            GetParentKey = LabelConnectionDtl(rstRecords![workspace_id])
        Case Else
            ' A workspace object cannot have a parent key
            Err.Raise vbObjectError + errInvalidNodeType, _
                mstrModuleName & "GetParentKey", _
                LoadResString(errInvalidNodeType)
            
    End Select
    
    Exit Function
    
GetParentKeyErr:

    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "GetParentKey"
    Err.Raise vbObjectError + errGetParentKeyFailed, _
            gstrSource, _
            LoadResString(errGetParentKeyFailed)
    
End Function
Public Function CopyWorkspace( _
        ByVal NodeIdentifier As Long, _
        ByVal NodeKey As String) As Object
    ' Returns an object populated with the data for the passed
    ' in workspace identifier
    
    Dim cNewWorkspace As cWorkspace
    
    On Error GoTo CopyWorkspaceErr
    gstrSource = mstrModuleName & "CopyWorkspace"

    Set cNewWorkspace = New cWorkspace
    Set cNewWorkspace.WorkDatabase = dbsAttTool
    cNewWorkspace.WorkspaceId = NodeIdentifier
    
    ' We do not store a collection of workspace objects,
    ' since it only has a name property - derive it
    cNewWorkspace.WorkspaceName = _
            FilterWorkspaceName( _
            frmMain.treStepView.Nodes(NodeKey).Text)
            
    Set CopyWorkspace = cNewWorkspace
    
    Exit Function
    
CopyWorkspaceErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "CopyWorkspace"
    On Error GoTo 0
    Set cNewWorkspace = Nothing
    Set CopyWorkspace = Nothing
    Err.Raise vbObjectError + errCopyWorkspaceFailed, _
            gstrSource, _
            LoadResString(errCopyWorkspaceFailed)
    
End Function
Public Sub CreateWsp(ByVal cNewWorkspace As cWorkspace)
    ' Creates the workspace object passed in - it must be
    ' initialized with the user input before this function is
    ' called
    
    Dim strWspName As String
    Dim lngAppend As Long
    
    On Error GoTo CreateWorkspaceErr

    strWspName = cNewWorkspace.WorkspaceName
    
    ' Control will go to the error handler if this call errors out
    ' A unique identifier for the workspace will be generated
    ' by the add function
    cNewWorkspace.AddWorkspace
    
    ' Create built in parameters
    Call CreateBuiltInParameter(cNewWorkspace.WorkspaceId, PARAM_DEFAULT_DIR, PARAM_DEFAULT_DIR_DESC)
    Call CreateBuiltInParameter(cNewWorkspace.WorkspaceId, PARAM_RUN_ID, PARAM_RUN_ID_DESC)
    Call CreateBuiltInParameter(cNewWorkspace.WorkspaceId, PARAM_OUTPUT_DIR, PARAM_OUTPUT_DIR_DESC)
    
    ' The workspace values are added to the tree view by the
    ' open workspace procedure
    
    ' Load all labels for the new workspace - this will
    ' also load the workspace node in the list view
    Call OpenWorkspace(cNewWorkspace.WorkspaceName, cNewWorkspace.WorkspaceId)

    Exit Sub
    
CreateWorkspaceErr:
    ' If the error raised by the add function is due to a duplication
    ' of the workspace name, then try to generate a unique name
    If Err.Number - vbObjectError = errDuplicateWorkspaceName Then
        lngAppend = lngAppend + 1
        cNewWorkspace.WorkspaceName = strWspName & _
                Trim$(Str(lngAppend))
        ' Try to insert the workspace record again
        Resume
    End If
    
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errCreateWorkspaceFailed, _
            mstrModuleName & "CreateWorkspace", _
            LoadResString(errCreateWorkspaceFailed)
    
End Sub

Private Sub DeleteParametersInWorkspace(lngWorkspaceId As Long)
    ' Deletes all the parameter and connection records for the workspace
    
    Dim nodChild As Node
    Dim strParameterLabel As String
    
    On Error GoTo DelParamsInWspErr

    strParameterLabel = LabelParameter(lngWorkspaceId)
    DelChildParamNodes (strParameterLabel)

    strParameterLabel = LabelBuiltIns(lngWorkspaceId)
    DelChildParamNodes (strParameterLabel)
    
    strParameterLabel = LabelAppExtension(lngWorkspaceId)
    DelChildParamNodes (strParameterLabel)
    
    strParameterLabel = LabelParamConnection(lngWorkspaceId)
    ' While more connection nodes exist
    While frmMain.treStepView.Nodes(strParameterLabel).Children > 0
        Set nodChild = frmMain.treStepView.Nodes(strParameterLabel).Child
        
        ' Call a procedure to carry out the processing necessary
        ' to delete the connection from the db and the loaded workspace
        Call DeleteConnectionRecord(ConnectionKey:=nodChild.Key)
    Wend

    Exit Sub
    
DelParamsInWspErr:
    gstrSource = mstrModuleName & "DeleteParametersInWorkspace"
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errDelParamsInWspFailed, _
            gstrSource, _
            LoadResString(errDelParamsInWspFailed)
    
End Sub
Private Sub DeleteConnDtlsInWsp(lngWorkspaceId As Long)
    ' Deletes all the connection_dtl records for the workspace
    
    Dim nodChild As Node
    Dim sLabel As String
    
    On Error GoTo DeleteConnDtlsInWspErr

    sLabel = LabelConnectionDtl(lngWorkspaceId)
    ' While more connection nodes exist
    While frmMain.treStepView.Nodes(sLabel).Children > 0
        Set nodChild = frmMain.treStepView.Nodes(sLabel).Child
        
        ' Call a procedure to carry out the processing necessary
        ' to delete the connection_dtl from the db and the loaded workspace
        Call DeleteConnDtlRecord(ConnDtlKey:=nodChild.Key)
    Wend

    Exit Sub
    
DeleteConnDtlsInWspErr:
    gstrSource = mstrModuleName & "DeleteConnDtlsInWsp"
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errDelParamsInWspFailed, _
            gstrSource, _
            LoadResString(errDelParamsInWspFailed)
    
End Sub

Private Sub DeleteWorkspaceRecords( _
    Optional ByVal WspToDelete As cWorkspace, _
    Optional ByVal WorkspaceId As Long = 0)
    ' Controls all the checks processing required during
    ' the deletion/archiving of a workspace record in the
    ' database

    Dim cDeleteWorkspace As cWorkspace
    Dim blnArchived As Boolean
    Dim strStepLabel As String
    
    On Error GoTo DeleteWorkspaceRecordsErr
    gstrSource = mstrModuleName & "DeleteWorkspaceRecords"
    
    If Not WspToDelete Is Nothing Then
        Set cDeleteWorkspace = WspToDelete
        WorkspaceId = cDeleteWorkspace.WorkspaceId
    ElseIf WorkspaceId > 0 Then
        Set cDeleteWorkspace = New cWorkspace
        Set cDeleteWorkspace.WorkDatabase = dbsAttTool
        cDeleteWorkspace.WorkspaceId = WorkspaceId
        cDeleteWorkspace.WorkspaceName = _
                FilterWorkspaceName(frmMain.treStepView.Nodes( _
                MakeKeyValid(WorkspaceId, gintWorkspace)).Text)
    Else
        On Error GoTo 0
        Err.Raise vbObjectError + errMandatoryParameterMissing, _
                gstrSource, _
                LoadResString(errMandatoryParameterMissing)
    End If
   
    If RunExistsForWorkspace(cDeleteWorkspace.WorkspaceId) Then
        ' Since a run exists for the workspace, it will not
        ' be deleted, but the record will be marked as archived
        cDeleteWorkspace.ArchivedFlag = True
        cDeleteWorkspace.ModifyWorkspace
        
        blnArchived = True
    Else
        ' Delete the workspace record alongwith all associated
        ' step, step-constraint and parameter records
        cDeleteWorkspace.DeleteWorkspace
        
        blnArchived = False
    End If
    
    ' Call a procedure to delete all parameter records for the
    ' workspace
    Call DeleteParametersInWorkspace(WorkspaceId)
    
    ' Call a procedure to delete all connection_dtls in the workspace
    Call DeleteConnDtlsInWsp(WorkspaceId)
    
    strStepLabel = LabelStep(WorkspaceId)
    ' Call a procedure to delete all step records for the
    ' workspace - the root node passed in will be set to the
    ' Steps Label
    Call DeleteSubSteps(strStepLabel, blnArchived)
        
    strStepLabel = LabelGlobalStep(WorkspaceId)
    ' Call a procedure to delete all global steps for the
    ' workspace - the root node passed in will be set to the
    ' Global Steps Label
    Call DeleteSubSteps(strStepLabel, blnArchived)
    
    ' Delete Workspace node in the list and tree view controls
    Call DeleteViewNode(frmMain.treStepView, _
        frmMain.lvwStepView, _
        cDeleteWorkspace.WorkspaceId, _
        gintWorkspace, WorkspaceId)
        
    ' Call a procedure to commit all the changes to the db
    Call SaveWorkspaceRecords(WorkspaceId)
    
    Set cDeleteWorkspace = Nothing
    
    Exit Sub
    
DeleteWorkspaceRecordsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "DeleteWorkspaceRecords"
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteWorkspaceRecordsFailed, _
            gstrSource, _
            LoadResString(errDeleteWorkspaceRecordsFailed)
    
End Sub
Public Function FilterWorkspaceName(strNodeText As String) As String
    ' Filters the workspace name from the node text
    FilterWorkspaceName = Mid(strNodeText, Len(mstrWorkspaceLabelText) + 1)
    
End Function

Public Sub LoadLabelsInTree(treStepView As TreeView, _
    lngWorkspaceId As Long, strWorkspaceName As String)

    ' We use a numbering scheme while loading the tree view with
    ' all node data, since it needs a unique key and we want to
    ' use the key to identify the data it contains.
    ' The general scheme is to concatenate a Label with the Identifier
    ' e.g A Global Step Node will have the Label, S and the Step Id
    ' concatenated to form the unique key
    ' The label identifier is a little different 'coz it does not
    ' have a unique identifier (primary key) that goes into
    ' the database. So, constants have been defined for the different
    ' label types. An exhaustive list of these values :
    ' Constant Name      : Label Id: Label String
    ' gintGlobalsLabel   : 11      : Globals
    ' gintGlobalStepLabel: 13      : Global Steps
    ' gintStepLabel      : 16      : Steps
    ' Module-level constants have been defined for these values
    ' The MakeKeyValid function is used to return unique identifiers
    ' for the different labels. The Identifier passed to it is the
    ' same as the node type in this case.
        
    Dim nodCurLevelNode As Node
    Dim strTempKey As String
    Dim intImageIndex As Integer
            
    ' Initializes the tree with the different nodes that it can
    ' contain.
    
    On Error GoTo LoadLabelsInTreeErr
    
    ' The top level node will display all open workspaces
    ' Add a workspace node - note that this is not a label!
    ' All subsequent nodes will be added relative to this node.
    ' Since we do not specify a relative node, the new node will
    ' be added in the last position of the top node hierarchy
    ' by default.
    Set nodCurLevelNode = treStepView.Nodes.Add(, , _
            MakeKeyValid(lngWorkspaceId, gintWorkspace), _
            mstrWorkspaceLabelText & strWorkspaceName, _
            gintImageWorkspaceClosed)
    intImageIndex = GetExpandedIcon(gintWorkspace)
    nodCurLevelNode.ExpandedImage = intImageIndex
    
    strTempKey = nodCurLevelNode.Key
    
    ' Retrieve the image index for all the remaining label nodes
    intImageIndex = GetExpandedIcon(gintGlobalsLabel)

    ' The Workspaces node displays global and step-specific information
    Set nodCurLevelNode = treStepView.Nodes.Add(strTempKey, _
            tvwChild, _
            LabelGlobals(lngWorkspaceId), _
            mstrGlobalsLabelText, _
            gintImageLabelClosed)
    nodCurLevelNode.ExpandedImage = intImageIndex
    strTempKey = nodCurLevelNode.Key
    ' Load the steps label next to the Globals node
    Set nodCurLevelNode = treStepView.Nodes.Add(strTempKey, _
            tvwNext, _
            LabelStep(lngWorkspaceId), _
            mstrStepLabelText, _
            gintImageLabelClosed)
    nodCurLevelNode.ExpandedImage = intImageIndex

    ' Similarly, the Globals node displays Global Parameters, Connection Strings,
    ' Connections, Application Extensions, built in parameters and Global steps.
    ' Add these as children of the Globals node
    Set nodCurLevelNode = treStepView.Nodes.Add(strTempKey, _
            tvwChild, _
            LabelBuiltIns(lngWorkspaceId), _
            mstrBuiltInLabelText, _
            gintImageLabelClosed)
    nodCurLevelNode.ExpandedImage = intImageIndex
    
    Set nodCurLevelNode = treStepView.Nodes.Add(strTempKey, _
            tvwChild, _
            LabelParameter(lngWorkspaceId), _
            mstrParameterLabelText, _
            gintImageLabelClosed)
    nodCurLevelNode.ExpandedImage = intImageIndex
    
    Set nodCurLevelNode = treStepView.Nodes.Add(strTempKey, _
            tvwChild, _
            LabelParamConnection(lngWorkspaceId), _
            mstrConnectStringLabelText, _
            gintImageLabelClosed)
    nodCurLevelNode.ExpandedImage = intImageIndex
    
    Set nodCurLevelNode = treStepView.Nodes.Add(strTempKey, _
            tvwChild, _
            LabelConnectionDtl(lngWorkspaceId), _
            mstrConnectDtlLabelText, _
            gintImageLabelClosed)
    nodCurLevelNode.ExpandedImage = intImageIndex
    
    Set nodCurLevelNode = treStepView.Nodes.Add(strTempKey, _
            tvwChild, _
            LabelAppExtension(lngWorkspaceId), _
            mstrApplicationLabelText, _
            gintImageLabelClosed)
    nodCurLevelNode.ExpandedImage = intImageIndex
    
    Set nodCurLevelNode = treStepView.Nodes.Add(strTempKey, _
            tvwChild, _
            LabelGlobalStep(lngWorkspaceId), _
            mstrGlobalStepLabelText, _
            gintImageLabelClosed)
    nodCurLevelNode.ExpandedImage = intImageIndex
    
    Exit Sub
    
LoadLabelsInTreeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "LoadLabelsInTree"
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadLabelsInTreeFailed, _
            gstrSource, _
            LoadResString(errLoadLabelsInTreeFailed)

End Sub
Public Sub OpenWorkspace( _
    Optional strWorkspaceName As String, _
    Optional ByVal lngWorkspaceId As Long = 0)

    ' This procedure calls routines to load the data for the
    ' passed in workspace into the tree view control. It takes
    ' either the workspace id or the workspace name as a
    ' parameter
    
    On Error GoTo OpenWorkspaceErr
    
    If lngWorkspaceId = 0 Then
        ' Retrieve the workspace id if it hasn't been specified
        lngWorkspaceId = CLng(GetWorkspaceDetails(WorkspaceName:=strWorkspaceName))
    End If
    
    ' Check if the data for the workspace has already
    ' been loaded
    If Not CheckWorkspaceOpen(lngWorkspaceId) Then
    
        If lngWorkspaceId > 0 Then
            ' Call a procedure to load the workspace details
            ' corresponding to the workspace
            Call PopulateWorkspaceDetails(lngWorkspaceId, _
                    gcSteps, gcParameters, gcConstraints, gcConnections, _
                    gcConnDtls, strWorkspaceName)
                    
            Call ModifyToolsMenu(lngWorkspaceId)
        End If
    Else
        ShowError errWorkspaceAlreadyOpen
    End If
        
    Exit Sub
    
OpenWorkspaceErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "OpenWorkspace"
    ShowError errOpenWorkspaceFailed
    On Error GoTo 0
    Err.Raise vbObjectError + errOpenWorkspaceFailed, _
            gstrSource, _
            LoadResString(errOpenWorkspaceFailed)
    
End Sub
Public Sub PopulateWorkspaceDetails(lngWorkspaceId As Long, _
        cStepsCol As cArrSteps, _
        cParamsCol As cArrParameters, _
        cConsCol As cArrConstraints, _
        cConns As cConnections, _
        cCDtls As cConnDtls, _
        Optional strWorkspaceName As String = gstrEmptyString)
    
    Dim rstWspSteps As Recordset
    Dim qySteps As DAO.QueryDef
    Dim rstWspParams As Recordset
    Dim qyParams As DAO.QueryDef
    Dim rstWspConns As Recordset
    Dim qyConns As DAO.QueryDef
    Dim rstWspConnDtls As Recordset
    Dim qyConnDtls As DAO.QueryDef
    
    On Error GoTo PopulateWorkspaceDetailsErr
    
    ' Read all the step and parameter data for the workspace
    Call ReadWorkspaceData(lngWorkspaceId, cStepsCol, _
            cParamsCol, cConsCol, cConns, cCDtls, rstWspSteps, _
            qySteps, rstWspParams, qyParams, rstWspConns, qyConns, _
            rstWspConnDtls, qyConnDtls)
            
    If strWorkspaceName = gstrEmptyString Then
        strWorkspaceName = CStr(GetWorkspaceDetails(lngWorkspaceId))
    End If

    ' First, load all the labels
    Call LoadLabelsInTree(frmMain.treStepView, lngWorkspaceId, strWorkspaceName)
    
    ' Load all the global, manager and worker steps in the workspace
    ' Here the type mintStepRecord will be used to indicate both
    ' non-global and global steps
    Call LoadRecordsetInTreeView(frmMain.treStepView, rstWspSteps, mintStepRecord)
        
    ' Load all the workspace parameters in the tree view
    Call LoadRecordsetInTreeView(frmMain.treStepView, rstWspParams, mintParameterRecord)
    
    ' Load all the connection strings in the tree view
    Call LoadRecordsetInTreeView(frmMain.treStepView, rstWspConns, gintNodeParamConnection)
    
    ' Load all the connections in the tree view
    Call LoadRecordsetInTreeView(frmMain.treStepView, rstWspConnDtls, gintConnectionDtl)
    
    Call ClearRecordset(rstWspParams)
    qyParams.Close
    Call ClearRecordset(rstWspSteps)
    qySteps.Close
    Call ClearRecordset(rstWspConns)
    qyConns.Close
    Call ClearRecordset(rstWspConnDtls)
    qyConnDtls.Close
    
    Exit Sub
    
PopulateWorkspaceDetailsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "PopulateWorkspaceDetails"
    Err.Raise vbObjectError + errPopulateWspDetailsFailed, _
            gstrSource, _
            LoadResString(errPopulateWspDetailsFailed)
    
End Sub
Public Sub ClearRecordset(rstStepsInWorkSpace As Recordset)

    On Error GoTo ClearRecordsetErr
    
    rstStepsInWorkSpace.Close
    Exit Sub
    
ClearRecordsetErr:
    LogErrors Errors
    Resume Next
    
End Sub

Public Sub SaveWorkspace(Optional ByVal lngWorkspace As Long = 0)
    
    Dim lngWorkspaceId As Long
    
    On Error GoTo SaveWorkspaceErr
    
    If Not AnyWorkspaceLoaded() Then
        Exit Sub
    End If
    
    ' Modify the mousepointer to indicate busy
    ShowBusy
    
    If lngWorkspace = 0 Then
        lngWorkspaceId = WorkspaceSelected
    Else
        lngWorkspaceId = lngWorkspace
    End If
    
    Call SaveWorkspaceRecords(lngWorkspaceId)
    
    ' Set the workspace tag to indicate that all changes have
    ' been saved
    Call WspCommitted(lngWorkspaceId)
    
    ' Reset the mousepointer
    ShowFree
    
    Exit Sub
    
SaveWorkspaceErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Reset the mousepointer
    ShowFree
    
    ' Display an error to the user
    gstrSource = mstrModuleName & "SaveWorkspace"
    ShowError errSaveWorkspaceFailed
    
End Sub

Public Function CheckWorkspaceOpen(lngWorkspaceId As Long) As Boolean

    ' Returns True if the passed in workspace has already
    ' been loaded into the tree view control
    Dim nodWorkspaceNode As Node
    Dim intIndex As Integer

    On Error GoTo CheckWorkspaceOpenErr
    
    CheckWorkspaceOpen = False
    
    Set nodWorkspaceNode = GetRootNode(frmMain.treStepView)
    If nodWorkspaceNode Is Nothing Then
        Exit Function
    End If
    
    If nodWorkspaceNode.Key = MakeKeyValid(lngWorkspaceId, gintWorkspace) Then
        CheckWorkspaceOpen = True
    Else
        intIndex = nodWorkspaceNode.LastSibling.Index
        
        Do While nodWorkspaceNode.Index <> intIndex
            Set nodWorkspaceNode = nodWorkspaceNode.Next
            If nodWorkspaceNode.Key = MakeKeyValid(lngWorkspaceId, gintWorkspace) Then
                CheckWorkspaceOpen = True
            End If
        Loop
    End If
    
    Exit Function
    
CheckWorkspaceOpenErr:
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "CheckWorkspaceOpen"
    On Error GoTo 0
    Err.Raise vbObjectError + errCheckWorkspaceOpenFailed, _
            gstrSource, _
            LoadResString(errCheckWorkspaceOpenFailed)

End Function
Private Sub SaveWorkspaceRecords(ByVal lngWorkspaceId As Long)
    
    Dim prpInTrans As Property
    Dim strInTrans As String
    
    On Error GoTo SaveWorkspaceRecordsErr
    gstrSource = mstrModuleName & "SaveWorkspaceRecords"
    
    ' Create a property that will indicate whether a
    ' transaction is open in the workspace
    strInTrans = "InTransaction"
    Set prpInTrans = dbsAttTool.CreateProperty(strInTrans, _
            dbBoolean, False)
    ' Append this property to the workspace
    dbsAttTool.Properties.Append prpInTrans
    ' Set the default value for the property to False
    dbsAttTool.Properties(strInTrans).Value = False
    
    If lngWorkspaceId > 0 Then
        ' Beginning of the transaction
        wrkJet.BeginTrans
        
        ' Set the InTransaction property to True
        dbsAttTool.Properties(strInTrans).Value = True
        
        ' Call a procedure to save all the parameters for the workspace
        Call gcParameters.SaveParametersInWsp(lngWorkspaceId)
        
        ' Call a procedure to save all connections in the workspace
        Call gcConnections.SaveConnectionsInWsp(lngWorkspaceId)
        
        ' Call a procedure to save all connection details in the workspace
        Call gcConnDtls.SaveConnDtlsInWsp(lngWorkspaceId)
        
        ' Call a procedure to save all the steps for the workspace
        Call gcSteps.SaveStepsInWsp(lngWorkspaceId)
        
        ' Call a procedure to save all the step constraints for the workspace
        Call gcConstraints.SaveWspConstraints(lngWorkspaceId)
        
        ' If we are through to this point, we can commit the transaction
        wrkJet.CommitTrans
        
        ' Set the InTransaction property to end of transaction
        dbsAttTool.Properties(strInTrans).Value = False
    End If
    
    dbsAttTool.Properties.Delete strInTrans
    Set prpInTrans = Nothing

    Exit Sub
    
SaveWorkspaceRecordsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    
    ' Use the InTransaction property to indicate whether a
    ' rollback is needed
    If dbsAttTool.Properties(strInTrans).Value Then
        ' Rollback any changes we have made
        wrkJet.Rollback
    End If
    dbsAttTool.Properties.Delete strInTrans
    
    gstrSource = mstrModuleName & "SaveWorkspaceRecords"
    On Error GoTo 0
    Err.Raise vbObjectError + errSaveWorkspaceFailed, _
        gstrSource, LoadResString(errSaveWorkspaceFailed)

End Sub
Public Sub WorkspaceClose(Optional ByVal lngWorkspaceId As Long = 0)

    ' Closes a workspace or unloads it.
    
    Dim strWorkspaceKey As String
    Dim lngIdentifier As Long
    
    On Error GoTo WorkspaceCloseErr
    
    If Not AnyWorkspaceLoaded() Then
        Exit Sub
    End If
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    If lngWorkspaceId = 0 Then
        lngIdentifier = WorkspaceSelected
    Else
        lngIdentifier = lngWorkspaceId
    End If
    
    strWorkspaceKey = MakeKeyValid(lngIdentifier, gintWorkspace)
    
    If lngIdentifier <> 0 Then
        If WorkspaceSelected(frmMain.lvwStepView.Tag) = lngIdentifier Then
            frmMain.lvwStepView.Tag = gstrEmptyString
        End If
        
        ' Save all pending changes
        Call ConfirmAndSaveChanges(lngIdentifier)
    
        ' Remove all the nodes from the workspace tree
        Call RemoveSubTree(frmMain.treStepView, strWorkspaceKey, frmMain.lvwStepView)
    
        Call ClearToolsMenu(lngIdentifier)
    End If
        
    ' Reset the mousepointer
    Call ShowFree
    
    Exit Sub
    
WorkspaceCloseErr:
    ' Log the VB error code
    LogErrors Errors
    ' Reset the mousepointer
    Call ShowFree
    gstrSource = mstrModuleName & "WorkspaceClose"
    ShowError errUnableToCloseWorkspace

End Sub
Public Sub CloseOpenWorkspaces()

    ' Closes all open workspaces - called when the user quits the application
    Dim nodWspNode As Node
    Dim nodNextNode As Node
    
    On Error GoTo CloseOpenWorkspacesErr
    
    Set nodWspNode = GetRootNode(frmMain.treStepView)
    
    Do While Not nodWspNode Is Nothing
        ' Need to determine the next node before we do a close
        ' workspace, since that deletes the workspace node
        If nodWspNode.Key <> nodWspNode.LastSibling.Key Then
            Set nodNextNode = nodWspNode.Next
        Else
            Set nodNextNode = Nothing
        End If
        
        Call WorkspaceClose(MakeIdentifierValid(nodWspNode.Key))
        Set nodWspNode = nodNextNode
    Loop
    
    ' Call a procedure to close the database if it is open
    Call CloseDatabase
    
    ' Delete all collections class instances
    Set gcSteps = Nothing
    Set gcParameters = Nothing
    Set gcConstraints = Nothing
    Set gcConnections = Nothing
    Set gcConnDtls = Nothing
    
    Exit Sub
    
CloseOpenWorkspacesErr:
    ' Log the VB error code
    LogErrors Errors
    ' Reset the mousepointer
    Call ShowFree
    gstrSource = mstrModuleName & "CloseOpenWorkspaces"
    ShowError errUnableToCloseWorkspace

End Sub
Public Sub WorkspaceModify()

    Dim lngWorkspaceId As Long
    Dim cWspToModify As cWorkspace
    
    On Error GoTo WorkspaceModifyErr

    Set cWspToModify = New cWorkspace
    
    ' Save updated workspace name to the db
    Set cWspToModify.WorkDatabase = dbsAttTool
    
    ' Initialize the workspace id of the class from the
    ' hidden label on the workspace form
    cWspToModify.WorkspaceId = Val(frmWorkspace.lblWorkspaceId)
    cWspToModify.WorkspaceName = frmWorkspace.txtWorkspaceName
    cWspToModify.ArchivedFlag = False
    ' Call the modify function of the workspace class
    cWspToModify.ModifyWorkspace
    
    ' Modify Workspace values in the tree and list view controls
    Call UpdateView(frmMain.treStepView, frmMain.lvwStepView, _
        cWspToModify.WorkspaceId, _
        mstrWorkspaceLabelText & cWspToModify.WorkspaceName, _
        cWspToModify, gintWorkspace)
    
    Set cWspToModify = Nothing
    
    ' Now that we're done, unload the Workspaces form
    Unload frmWorkspace
    Exit Sub
    
WorkspaceModifyErr:
    LogErrors Errors
    ' Let the user enter a valid workspace name
    gstrSource = mstrModuleName & "WorkspaceModify"
    Call ClearFormAndSetFocus(frmWorkspace, _
        frmWorkspace.txtWorkspaceName)
    ' Reset the mousepointer before displaying an error
    Call ShowFree
    ShowError errUnableToModifyWorkspace
    
End Sub
Public Sub WorkspaceNew()

    Dim lngWorkspaceId As Long
    Dim cNewWorkspace As cWorkspace
    
    On Error GoTo WorkspaceNewErr

    ' All checks for validity of the Workspace name will be done
    ' by the workspace class
        
    ' Initialize the values of the workspace being created
    Set cNewWorkspace = New cWorkspace
    
    ' Save Workspace values to the db
    Set cNewWorkspace.WorkDatabase = dbsAttTool
    cNewWorkspace.WorkspaceName = frmWorkspace.txtWorkspaceName
    
    ' Call a procedure to do all the processing necessary to
    ' add this workspace to the database and to the screen
    Call CreateWsp(cNewWorkspace)
    
    Set cNewWorkspace = Nothing
    
    ' Now that we're done, clear and unload the Workspaces form
    Call ClearForm(frmWorkspace)
    Unload frmWorkspace
    
    Exit Sub
    
WorkspaceNewErr:
    LogErrors Errors
    ' Give the user a chance to enter a valid workspace name
    gstrSource = mstrModuleName & "WorkspaceNew"
    Call ClearFormAndSetFocus(frmWorkspace, _
        frmWorkspace.txtWorkspaceName)
    ' Reset the mousepointer before displaying an error
    Call ShowFree
    ShowError errUnableToCreateWorkspace
    
End Sub
Sub WorkspaceDelete(Optional ByVal DeleteWorkspace As cWorkspace, _
        Optional ByVal lngWorkspaceId As Long = 0)
    
    Dim lngIdentifier As Long
    
    On Error GoTo WorkspaceDeleteErr

    If DeleteWorkspace Is Nothing Then
        If lngWorkspaceId = 0 Then
            lngIdentifier = IdentifierSelected
        Else
            lngIdentifier = lngWorkspaceId
        End If
    Else
        lngIdentifier = DeleteWorkspace.WorkspaceId
    End If
    
    ' Prompt the user whether he ReallY wants to delete the
    ' workspace.
    If Confirm(conWspDelete, contWspDelete) Then
        If lngIdentifier = 0 Then
            ' No selection has been made by the user,
            BugAssert False, "WorkspaceDelete called when an item wasn't selected!"
        Else
            Call DeleteWorkspaceRecords(DeleteWorkspace, lngIdentifier)
        End If
    End If
    
    Exit Sub
    
WorkspaceDeleteErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "WorkspaceDelete"
    On Error GoTo 0
    Err.Raise vbObjectError + errWorkspaceDeleteFailed, gstrSource, _
            LoadResString(errWorkspaceDeleteFailed)
    
End Sub
Private Function RunExistsForWorkspace( _
    ByVal lngWorkspaceIdentifier As Long) As Boolean
    ' Check if a run exists for the Workspace
   
    Dim rstRun As Recordset
    Dim qyRun As DAO.QueryDef
    Dim strSql As String

    On Error GoTo RunExistsForWorkspaceErr
    gstrSource = "RunExistsForWorkspace"
    
    ' Create a recordset object containing a count of all
    ' runs for the workspace
    strSql = "Select count(*) as run_count " & _
        " from run_header " & _
        " where workspace_id = [w_id] "
    Set qyRun = dbsAttTool.CreateQueryDef(gstrEmptyString, strSql)
    qyRun.Parameters("w_id").Value = lngWorkspaceIdentifier
    
    Set rstRun = qyRun.OpenRecordset(dbOpenForwardOnly)
    
    If rstRun![run_count] > 0 Then
        RunExistsForWorkspace = True
    Else
        RunExistsForWorkspace = False
    End If
    rstRun.Close
    qyRun.Close

    Exit Function
    
RunExistsForWorkspaceErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "RunExistsForWorkspace"
    On Error GoTo 0
    Err.Raise vbObjectError + errRunExistsForWorkspaceFailed, _
        gstrSource, LoadResString(errRunExistsForWorkspaceFailed)

End Function
Public Sub WorkspaceOk()

    ' Errors will be handled by the called functions
    On Error Resume Next
    
    ' Set the mousepointer to indicate Busy
    Call ShowBusy
    
    If frmWorkspace.Tag = gstrPropertiesMode Then
        Unload frmWorkspace
    ElseIf frmWorkspace.Tag = gstrInsertMode Then
        Call WorkspaceNew
    Else
        Call WorkspaceModify
    End If
    
    ' Reset the mousepointer
    Call ShowFree

End Sub
Sub WorkspaceOpenOk()

    Dim intIndex As Integer
    
    On Error GoTo WorkspaceOpenOkErr
    
    ShowBusy
    
    ' Step through each element in the workspaces list and
    ' determine if it has been selected
    For intIndex = 0 To frmWorkspaceOpen.lstWorkspaces.ListCount - 1
        
        If frmWorkspaceOpen.lstWorkspaces.Selected(intIndex) Then
            Select Case frmWorkspaceOpen.Caption
                Case msOpenWsp
                    ' Call a procedure to load the workspace data into
                    ' the tree view. The workspace id is stored in the
                    ' Itemdata property of the list, whereas the workspace
                    ' name is displayed to the user
                    Call OpenWorkspace(frmWorkspaceOpen.lstWorkspaces.List(intIndex), _
                        Val(frmWorkspaceOpen.lstWorkspaces.ItemData(intIndex)))
                Case msImportWsp
                    Call DoImport(Val(frmWorkspaceOpen.lstWorkspaces.ItemData(intIndex)))
                Case msExportWsp
                    Call DoExport(Val(frmWorkspaceOpen.lstWorkspaces.ItemData(intIndex)))
                Case msAppExtensionCaption
                    Call AppExtensionOK(frmWorkspaceOpen.lstWorkspaces.List(intIndex))
                Case Else
                    BugAssert True, "Invalid caption!"
            End Select
        End If
    
    Next intIndex
    
    ' Unload the form
    Unload frmWorkspaceOpen
    ShowFree
        
    Exit Sub
    
WorkspaceOpenOkErr:
    LogErrors Errors
    ' Errors if any, have already been displayed by the called
    ' functions
    ShowFree

End Sub

Public Sub ShowWorkspaceOpen()

    On Error GoTo ShowWorkspaceOpenErr
    
    If Not gblnDbOpen Then
        ' No database has been opened yet
        Exit Sub
    End If
    
    frmWorkspaceOpen.Caption = msOpenWsp
    
    Call ShowWorkspacesInDb(dbsAttTool)
    
    Exit Sub
    
ShowWorkspaceOpenErr:
    ' Log the VB error code
    LogErrors Errors
    gstrSource = mstrModuleName & "ShowWorkspaceOpen"
    ShowError errShowAllWorkspacesFailed

End Sub
Public Sub ShowWorkspaceImport(dbImport As Database)

    On Error GoTo ShowWorkspaceImportErr
    
    frmWorkspaceOpen.Caption = msImportWsp
    
    Call ShowWorkspacesInDb(dbImport)
    
    Exit Sub
    
ShowWorkspaceImportErr:
    ' Log the VB error code
    LogErrors Errors
    gstrSource = mstrModuleName & "ShowWorkspaceImport"
    ShowError errShowAllWorkspacesFailed

End Sub
Public Sub ShowWorkspaceExport(dbExport As Database)

    On Error GoTo ShowWorkspaceExportErr
    
    frmWorkspaceOpen.Caption = msExportWsp
    
    Call ShowWorkspacesInDb(dbExport)
    
    Exit Sub
    
ShowWorkspaceExportErr:
    ' Log the VB error code
    LogErrors Errors
    gstrSource = mstrModuleName & "ShowWorkspaceExport"
    ShowError errShowAllWorkspacesFailed

End Sub

Sub ShowWorkspaceNew()
    
    On Error GoTo ShowWorkspaceNewErr
    
    Load frmWorkspace
    frmWorkspace.Tag = gstrInsertMode
    
    ' Reset the mousepointer
    ShowFree
    
    frmWorkspace.Show vbModal, frmMain
    Exit Sub
    
ShowWorkspaceNewErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "ShowWorkspaceNew"
    On Error GoTo 0
    Err.Raise vbObjectError + errShowWorkspaceNewFailed, _
        gstrSource, LoadResString(errShowWorkspaceNewFailed)

End Sub
Sub ShowWorkspaceModify(Optional ByVal lngIdentifier As Long = 0)
    
    Dim lngWorkspaceId As Long
    
    On Error GoTo ShowWorkspaceModifyErr
    
    Load frmWorkspace
    
    If lngIdentifier = 0 Then
        lngWorkspaceId = IdentifierSelected
    Else
        lngWorkspaceId = lngIdentifier
    End If
        
    If lngWorkspaceId = 0 Then
        Exit Sub
    End If
        
    ' We do not maintain a collection of workspace objects
    ' since it has just a name property, which can be stored
    ' in the tree view control. The id for the workspace
    ' being modified will be stored in a hidden field on
    ' the Workspace modify form
    frmWorkspace.lblWorkspaceId = Str(lngWorkspaceId)
    frmWorkspace.txtWorkspaceName = _
            FilterWorkspaceName(frmMain.treStepView.SelectedItem.Text)
    
    Call HighlightText(frmWorkspace.txtWorkspaceName)
    
    frmWorkspace.Tag = gstrUpdateMode
    
    ' Reset the mousepointer
    ShowFree
    
    frmWorkspace.Show vbModal, frmMain
    
    Exit Sub
    
ShowWorkspaceModifyErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "ShowWorkspaceModify"
    On Error GoTo 0
    Err.Raise vbObjectError + errShowWorkspaceModifyFailed, _
        gstrSource, LoadResString(errShowWorkspaceModifyFailed)

End Sub
