VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cVectorLng"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:        cVectorLng.cls
'               Microsoft TPC-H Kit Ver. 2.7.0-1005
'               Copyright Microsoft, 2008
'               All Rights Reserved
'
'
'  PURPOSE:     This class implements an array of longs.
'  Contact:     Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this class
Private mstrSource As String
Private Const mstrModuleName As String = "cVectorLng."

' Array counter
Private mlngCount As Long
Private mcarrItems() As Long

Public Sub Add(ByVal lngItem As Long)
    ' Adds the passed in long variable to the array

    On Error GoTo AddErr
    
    ReDim Preserve mcarrItems(mlngCount)
    
    ' Set the newly added element in the array to the
    ' passed in variable
    mcarrItems(mlngCount) = lngItem
    mlngCount = mlngCount + 1
    
    Exit Sub
    
AddErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "Add"
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadInArrayFailed, _
            mstrSource, _
            LoadResString(errLoadInArrayFailed)
        
End Sub
Public Sub Clear()
        
    ' Clear the array
    ReDim mcarrItems(0)

End Sub

Public Sub Delete(Optional ByVal Position As Long = -1, _
        Optional ByVal Item As Long = -1)
    ' The user can opt to delete either a specific item in
    ' the list or the item at a specified position. If no
    ' parameters are passed in, we delete the element at
    ' position 0!
    
    Dim lngDelete As Long
    Dim lngIndex As Long
    
    On Error GoTo DeleteErr
    
    If Position = -1 Then
        ' Since we can never store an element at position -1,
        ' we can be sure that the user is trying to delete
        ' a given item
        lngDelete = Find(Item)
    Else
        lngDelete = Position
    End If
        
    If lngDelete < (mlngCount - 1) Then
        
        ' We want to maintain the order of all items in the
        ' array - so move all remaining elements in the array
        ' up by 1
        For lngIndex = lngDelete To mlngCount - 2
            MoveDown lngIndex
        Next lngIndex
        
    End If

    ' Delete the last Node from the array
    mlngCount = mlngCount - 1
    If mlngCount > 0 Then
        ReDim Preserve mcarrItems(0 To mlngCount - 1)
    Else
        ReDim mcarrItems(0)
    End If
    
    Exit Sub
    
DeleteErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "Delete"
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteArrayElementFailed, _
            mstrSource, _
            LoadResString(errDeleteArrayElementFailed)
        
End Sub
Public Function Find(ByVal Item As Long) As Long
    
    ' Returns the position at which the passed in value occurs
    ' in the array
    
    Dim lngIndex As Long
        
    On Error GoTo FindErr
    
    ' Find the element in the array to be deleted
    For lngIndex = 0 To mlngCount - 1
    
        If mcarrItems(lngIndex) = Item Then
            Find = lngIndex
            Exit Function
        End If
    
    Next lngIndex
    
    Find = -1
    
    Exit Function
    
FindErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "Find"
    On Error GoTo 0
    Err.Raise vbObjectError + errItemNotFound, mstrSource, _
        LoadResString(errItemNotFound)
        
End Function
Public Property Get Item(ByVal Position As Long) As Long
Attribute Item.VB_UserMemId = 0
    
    ' Returns the element at the passed in position in the array
    If Position >= 0 And Position < mlngCount Then
        Item = mcarrItems(Position)
    Else
        On Error GoTo 0
        Err.Raise vbObjectError + errItemDoesNotExist, mstrSource, _
            LoadResString(errItemDoesNotExist)
    End If
    
End Property
Public Property Let Item(ByVal Position As Long, _
        ByVal Value As Long)
    
    ' Returns the element at the passed in position in the array
    If Position >= 0 Then
        ' If the passed in position is outside the array
        ' bounds, then resize the array
        If Position >= mlngCount Then
            ReDim Preserve mcarrItems(Position)
            mlngCount = Position + 1
        End If
        
        ' Set the newly added element in the array to the
        ' passed in variable
        mcarrItems(Position) = Value
    Else
        On Error GoTo 0
        Err.Raise vbObjectError + errItemDoesNotExist, mstrSource, _
            LoadResString(errItemDoesNotExist)
    End If
    
End Property
Public Sub MoveUp(ByVal Position As Long)
    ' Moves the element at the passed in position up by 1
    
    Dim lngTemp As Long
    
    If Position > 0 And Position < mlngCount Then
        lngTemp = mcarrItems(Position)
        
        mcarrItems(Position) = mcarrItems(Position - 1)
        mcarrItems(Position - 1) = lngTemp
    End If
    
End Sub
Public Sub MoveDown(ByVal Position As Long)
    ' Moves the element at the passed in position down by 1
    
    Dim lngTemp As Long
    
    If Position >= 0 And Position < mlngCount - 1 Then
        lngTemp = mcarrItems(Position)
        
        mcarrItems(Position) = mcarrItems(Position + 1)
        mcarrItems(Position + 1) = lngTemp
    End If
    
End Sub

Public Function Count() As Long
    
    Count = mlngCount
    
End Function


Private Sub Class_Initialize()
    
    mlngCount = 0

End Sub


Private Sub Class_Terminate()
        
    Call Clear
    
End Sub

